note
	description: "Objects that `notify' its observers whenever a change occurs"
	author: "Jonathan S. Ostroff"
	date: "$March 24, 2002$"
	revision: "$Revision 1.0$"

class
	SUBJECT

create
	make

feature -- Initialization
	make
			-- initialize `observers'
		do
			create {LINKED_LIST[OBSERVER]} observers.make
		end

feature -- queries

	observers: LIST[OBSERVER]
		-- List of all observers attached to this subject

	is_attached_to(o: OBSERVER): BOOLEAN
			-- Is observer `o' attached to current observer?
		do
			Result := observers.has (o)
		ensure
			is_attached_defn: Result = observers.has(o)
		end


feature  -- Invoked by an OBSERVER

	attach (o: OBSERVER)
			-- Add `o' to the observers
		require
			not_yet_attached: not is_attached_to(o)
		do
			observers.extend(o)
		ensure
			is_attached: is_attached_to(o)
		end


	detach (o: OBSERVER)
			-- add `o' to the observers
		require
			currently_attached: is_attached_to(o)
		do
			observers.search(o)
			observers.remove
		ensure
			is_attached: not is_attached_to(o)
		end

feature -- invoked by a SUBJECT

	notify
			-- Send an `update' message to each observer
		do
			across
				observers as cursor
			loop
				cursor.item.update
			end
		ensure
			all_views_updated:
				across
					observers as cursor
				all
					cursor.item.up_to_date_with_subject
				end
		end

end -- class SUBJECT
